package com.cicdi.notify;

import com.cicdi.notify.template.Template;

import java.util.Map;

/**
 * 消息通知器
 *
 * @author xueye
 */
public interface Notifier<T extends Template> {

    /**
     * 获取通知器唯一标识
     *
     * @return 通知器唯一标识
     */
    String getNotifierId();

    /**
     * 获取通知类型
     *
     * @return 通知类型
     * @see NotifyType
     */
    NotifyType getType();

    /**
     * 获取服务商
     *
     * @return 通知服务商
     */
    Provider getProvider();

    /**
     * 指定通知模板ID进行发送
     * <p>用于一些固定模板的消息发送，例如验证码、定时消息</p>
     *
     * @param templateId 通知模板ID
     * @param context    模板上下文参数
     */
    void send(String templateId, Map<String, Object> context);

    /**
     * 指定通知模板进行发送
     * <p>提供一个通知模板进行消息发送，比较灵活，适用于一些一次性的消息推送<p/>
     *
     * @param template 通知模板
     * @param context  模板上下文参数
     */
    void send(T template, Map<String, Object> context);

    /**
     * 关闭通知器
     */
    void close();

    /**
     * 将此通知器转换成特定类型通知器
     *
     * @param type 目标类型类
     * @param <R>  目标类型
     * @return 目标类型通知器
     */
    default <R extends Notifier<T>> R unwrap(Class<R> type) {
        return type.cast(this);
    }

}
