package com.cicdi.notify.sms.aliyun;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.cicdi.notify.*;
import com.cicdi.notify.sms.SmsProvider;
import com.cicdi.notify.template.TemplateManager;


import java.util.Map;
import java.util.Objects;

public class AliyunSmsNotifier extends AbstractNotifier<AliyunSmsTemplate> {

    private final IAcsClient client;
    private final int connectTimeout = 1000;
    private final int readTimeout = 5000;

    private String notifierId;

    public String getNotifierId() {
        return notifierId;
    }

    private String domain = "dysmsapi.aliyuncs.com";
    private String regionId = "cn-hangzhou";

    public AliyunSmsNotifier(NotifierProperties profile, TemplateManager templateManager) {
        super(templateManager);
        Map<String, Object> config = profile.getConfiguration();
        DefaultProfile defaultProfile = DefaultProfile.getProfile(
                this.regionId = (String) Objects.requireNonNull(config.get("regionId"), "regionId不能为空"),
                (String) Objects.requireNonNull(config.get("accessKeyId"), "accessKeyId不能为空"),
                (String) Objects.requireNonNull(config.get("secret"), "secret不能为空")
        );
        this.client = new DefaultAcsClient(defaultProfile);
        this.domain = (String) config.getOrDefault("domain", domain);
        this.notifierId = profile.getId();
    }

    public AliyunSmsNotifier(IClientProfile profile, TemplateManager templateManager) {
        this(new DefaultAcsClient(profile), templateManager);
    }

    public AliyunSmsNotifier(IAcsClient client, TemplateManager templateManager) {
        super(templateManager);
        this.client = client;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.sms;
    }

    @Override
    public Provider getProvider() {
        return SmsProvider.aliyunSms;
    }

    @Override
    public void send(AliyunSmsTemplate template, Map<String, Object> context) {
        try {
            CommonRequest request = new CommonRequest();
            request.setSysMethod(MethodType.POST);
            request.setSysDomain(domain);
            request.setSysVersion("2017-05-25");
            request.setSysAction("SendSms");
            request.setSysConnectTimeout(connectTimeout);
            request.setSysReadTimeout(readTimeout);
            request.putQueryParameter("RegionId", regionId);
            request.putQueryParameter("PhoneNumbers", template.getPhoneNumber());
            request.putQueryParameter("SignName", template.getSignName());
            request.putQueryParameter("TemplateCode", template.getCode());
            request.putQueryParameter("TemplateParam", template.createTtsParam(context));

            CommonResponse response = client.getCommonResponse(request);

            JSONObject json = JSON.parseObject(response.getData());
            if (!"ok".equalsIgnoreCase(json.getString("Code"))) {
                // TODO 自定义异常
//                    return new BusinessException(json.getString("Message"), json.getString("Code"));
                throw new Exception("json.getString(\"Message\"), json.getString(\"Code\")");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void close() {
        client.shutdown();
    }
}
