package com.cicdi.notify.wechat;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cicdi.notify.AbstractNotifier;
import com.cicdi.notify.DefaultNotifyType;
import com.cicdi.notify.NotifyType;
import com.cicdi.notify.Provider;
import com.cicdi.notify.template.TemplateManager;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class WeixinCorpNotifier extends AbstractNotifier<WechatMessageTemplate> {

    private String accessToken;

    private long refreshTokenTime;

    private final long tokenTimeOut = Duration.ofSeconds(7000).toMillis();

    private static final String tokenApi = "https://qyapi.weixin.qq.com/cgi-bin/gettoken";

    private static final String notifyApi = "https://qyapi.weixin.qq.com/cgi-bin/message/send";

    private final WechatCorpProperties properties;

    private final String notifierId;

    @Override
    public String getNotifierId() {
        return notifierId;
    }


    public WeixinCorpNotifier(String id, WechatCorpProperties properties, TemplateManager templateManager) {
        super(templateManager);
        this.properties = properties;
        this.notifierId = id;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.wechat;
    }

    @Override
    public Provider getProvider() {
        return WechatProvider.corpMessage;
    }

    @Override
    public void send(WechatMessageTemplate template, Map<String, Object> context) {
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        HttpPost httpPost = new HttpPost(notifyApi + "?access_token=" + getToken());
        httpPost.setHeader("Content-Type", "application/json;charset=utf8");

        StringEntity stringEntity = new StringEntity(template.createJsonRequest(context), "UTF-8");
        httpPost.setEntity(stringEntity);

        try {
            CloseableHttpResponse response = httpClient.execute(httpPost);
            String result = EntityUtils.toString(response.getEntity());
            System.out.println(result);
            JSONObject jsonObject = JSON.parseObject(result);
            if ("0".equals(jsonObject.get("errcode").toString())) {
//                log.info("发送微信企业通知成功");
            } else {
//                log.warn("发送微信企业通知失败:{}", map);
//                throw new BusinessException("发送微信企业通知失败:" + map.get("errmsg"), code);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private String getToken() {
        if (System.currentTimeMillis() - refreshTokenTime > tokenTimeOut || accessToken == null) {
            return requestToken();
        }
        return accessToken;
    }

    private String requestToken() {
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("corpId", properties.getCorpId()));
        params.add(new BasicNameValuePair("corpSecret", properties.getCorpSecret()));

        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        try {
            HttpGet httpGet = new HttpGet(new URIBuilder(tokenApi).setParameters(params).build());
            CloseableHttpResponse response = httpClient.execute(httpGet);
            JSONObject responseJson = JSON.parseObject(EntityUtils.toString(response.getEntity()));
            if (responseJson.containsKey("access_token")) {
                String access_token = responseJson.get("access_token").toString();
                accessToken = access_token;
                refreshTokenTime = System.currentTimeMillis();
                return access_token;
            } else {
                // TODO 自定义异常
                // throw new BusinessException("获取Token失败:" + map.get("errmsg"), String.valueOf(map.get("errcode")));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public void close() {
        this.accessToken = null;
        this.refreshTokenTime = 0;
    }
}
